/*
 * Copyright (c) 2002-2004 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.kleopatra.forms.examples.bookmanager;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.jgoodies.binding.list.SelectionInList;

import de.kleopatra.forms.examples.data.Book;
import de.kleopatra.forms.examples.data.BookManager;
import de.kleopatra.forms.view.builder.Form;
import de.kleopatra.forms.view.builder.JFormDialog;

/**
 * Provides the models and Actions for managing and editing Books.
 * Works with an underlying BookManager that provides a ListModel
 * for the Books and operations to add, remove, and change a Book.
 * In other words, this class turns the raw data and operations
 * form the BookManager into a form usable in a user interface.<p>
 *
 * This model keeps the Books in a SelectionInList, refers to another
 * PresentationModel for editing the selected Book, and provides
 * Actions for the Book operations: add, remove and edit the selected Book.
 *
 * JW: Adjusted to use Forms as user interface: <p>
 * 
 * BookManagerForm describes the main selection, the operations to
 * add/remove/edit the selected Book are hooked as actions to the form's
 * buttons. <p>
 * 
 * BookEditorForm describes the book editor, it's opened in a JFormDialog
 * which hooks its ok/cancel methods to the form's ok/cancel buttons.
 * 
 * @author Karsten Lentzsch
 * @author Jeanette Winzenburg
 * @version $Revision: 1.3 $
 *
 * @see BookManager
 * @see com.jgoodies.binding.PresentationModel
 */
public final class BookManagerModel {

  /**
   * Holds the List of Books and provides operations
   * to create, add, remove and change a Book.
   */
  private final BookManager bookManager;

  /**
   * Holds the list of managed books plus a single selection.
   */
  private SelectionInList bookSelection;
  private Action newAction;
  private Action editAction;
  private Action deleteAction;
  private Form form;
  private JFormDialog editorDialog;

  /**
   * Constructs a BookManager for editing the given list of books.
   *
   * @param bookManager   the list of books to edit
   */
  public BookManagerModel(BookManager bookManager) {
    this.bookManager = bookManager;
    initModels();
    initEventHandling();
  }

//------------- accessing the Form

  /** pre: bookSelection != null.
   *
   * @return
   */
  public Form getForm() {
    if (form == null) {
      form = new BookManagerForm(bookSelection);
    }
    return form;
  }

//-------------------- Managing Books 
  private void addItem(Book bookToAdd) {
    bookManager.addItem(bookToAdd);
  }

  /**
   * Opens a BookEditorDialog for the given Book.
   *
   * @param book  the Book to be edited
   * @return true if the dialog has been canceled, false if accepted
   */
  private boolean openBookEditor(Book book) {
    JFormDialog dialog = getEditorDialog();
    dialog.setTarget(book);
    dialog.open();
    return dialog.hasBeenCanceled();
  }

  private JFormDialog getEditorDialog() {
    if (editorDialog == null) {
      editorDialog = new JFormDialog(null, new BookEditorForm());
      editorDialog.setTitle("Book Editor");
    }
    return editorDialog;
  }

  private Book createAndAddItem() {
    Book newBook = bookManager.createItem();
    boolean canceled = openBookEditor(newBook);
    if (!canceled) {
      addItem(newBook);
      return newBook;
    }
    return null;
  }

  private void editSelectedItem() {
    boolean canceled = openBookEditor(getSelectedItem());
    if (!canceled) {
      bookManager.markItemChanged(getSelectedIndex());
    }
  }

  private void deleteSelectedItem() {
    bookManager.removeItem(getSelectedItem());
  }

  private Book getSelectedItem() {
    return (Book) bookSelection.getSelection();
  }

  private int getSelectedIndex() {
    return bookSelection.getSelectionIndex();
  }

//-------------- Event Handling 
  private void updateActionEnablement() {
    boolean hasSelection = bookSelection.hasSelection();
    editAction.setEnabled(hasSelection);
    deleteAction.setEnabled(hasSelection);
  }

  private class SelectionEmptyHandler implements PropertyChangeListener {

    public void propertyChange(PropertyChangeEvent evt) {
      updateActionEnablement();
    }
  }

//-------------------------- init

  /**
   * Initializes the SelectionInList, Actions and Form.
   * Register the actions as handlers with the form.
   * To synchronize the Action enablement
   * with the selection state, we update the enablement now.
   */
  private void initModels() {
    bookSelection = new SelectionInList(bookManager.getManagedBooks());
    newAction = new NewAction();
    editAction = new EditAction();
    deleteAction = new DeleteAction();
    getForm().registerActionsAsHandlers(
        Arrays.asList(new String[]{ "newBook", "editBook", "deleteBook" }), 
        Arrays.asList(new Action[]{newAction, editAction, deleteAction }));
    updateActionEnablement();
  }

  /**
   * Initializes the event handling by just registering a
   * handler that updates the Action enablement if the
   * bookSelection's 'selectionEmpty' property changes.
   */
  private void initEventHandling() {
    bookSelection
      .addPropertyChangeListener(SelectionInList
        .PROPERTYNAME_SELECTION_EMPTY, new SelectionEmptyHandler());
  }

//------------------------ Actions
  private class NewAction extends AbstractAction {

    private NewAction() {
      super("New\u2026");
    }

    public void actionPerformed(ActionEvent e) {
      Object newBook = createAndAddItem();
      bookSelection.setSelection(newBook);
    }
  }

  private class EditAction extends AbstractAction {

    private EditAction() {
      super("Edit\u2026");
    }

    public void actionPerformed(ActionEvent e) {
      editSelectedItem();
    }
  }

  private class DeleteAction extends AbstractAction {

    private DeleteAction() {
      super("Delete");
    }

    public void actionPerformed(ActionEvent e) {
      deleteSelectedItem();
    }
  }
}
