/*
 * Copyright (c) 2002-2004 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

package de.kleopatra.forms.examples.data;

import java.util.List;

import javax.swing.ListModel;

import com.jgoodies.binding.list.ArrayListModel;

/**
 * Holds a List of Books and provides operations to add, delete and
 * change a Book. Such a manager is part of the domain layer.<p>
 * 
 * This manager holds the Books in an ArrayListModel, so we can
 * operate on a List and can expose it as a ListModel.
 * As an alternative, a higher-level presentation model, such as
 * the BookManagerModel could turn the List into a ListModel.
 * In the latter case, you would then need to fire the required
 * ListDataEvents.<p>
 * 
 * The BookManagerModel turns the List of Books and the operations
 * into a form that can be used in a user interface to display,
 * select, and edit Books.<p>
 * 
 * TODO: Demonstrate how to sort the books.  
 * 
 * @author Karsten Lentzsch
 * @version $Revision: 1.1 $
 * 
 * @see BookManagerModel
 */

public final class BookManager {
    
    /**
     * Holds the List of Books. Books are added and removed from this List.
     * The ObservableList implements ListModel, and so, we can directly
     * use this List for the UI and can observe changes.<p>
     * 
     * In a real world application this List may be held
     * by a BookManager, or may be kept in synch with a DataBase.
     */
    private final ArrayListModel managedBooks;
    

    // Instance Creation ******************************************************
    
    /**
     * Constructs a BookManager for the given list of Books.
     * 
     * @param books   the list of books to manage
     */
    public BookManager(List books) {
        this.managedBooks = new ArrayListModel(books);
    }
    
    
    // Exposing the ListModel of Books ****************************************
    
    public ListModel getManagedBooks() {
        return managedBooks;
    }
    
    
    // Managing Books *********************************************************
    
    /**
     * Creates and return a new Book.
     * 
     * @return the new Book
     */
    public Book createItem() {
        return new Book();
    }
    
    
    /**
     * Adds the given Book to the List of managed Books
     * and notifies observers of the managed books ListModel
     * about the change.
     * 
     * @param bookToAdd   the Book to add
     */
    public void addItem(Book bookToAdd) {
        managedBooks.add(bookToAdd);
    }
    
    
    /**
     * Notifies observers of the managed books ListModel about
     * a changed book at the given index
     * 
     * @param index   the index where the contents changed
     */
    public void markItemChanged(int index) {
        managedBooks.fireContentsChanged(index);
    }
    
    
    /**
     * Removes the given Book from the List of managed Books
     * and notifies observers of the managed books ListModel
     * about the change.
     * 
     * @param bookToRemove    the Book to remove
     */
    public void removeItem(Book bookToRemove) {
        managedBooks.remove(bookToRemove);
    }
    
}