/*
 * Copyright (c) 2002-2004 JGoodies Karsten Lentzsch. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of JGoodies Karsten Lentzsch nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

package de.kleopatra.forms.examples.data;

import java.text.DateFormat;
import java.util.*;

import com.jgoodies.binding.beans.Model;

/**
 * Describes a book and provides bound bean properties.
 * 
 * @author  Karsten Lentzsch
 * @version $Revision: 1.2 $
 */
public final class MusicCD extends Model {
    
    // Examples ***************************************************************
    
    public static final MusicCD CD1 = createExample1();
    public static final MusicCD CD2 = createExample2();
    public static final MusicCD CD3 = createExample3();
    public static final MusicCD CD4 = createExample4();
    
    public static final List CDS = Arrays.asList(new MusicCD[]{
            CD1, CD2, CD3, CD4});
    
    
    // Names of the Bound Bean Properties *************************************

    public static final String PROPERTYNAME_AUTHORS      = "authors";
    public static final String PROPERTYNAME_AVAILABLE    = "available";
    public static final String PROPERTYNAME_PRICE        = "price";
    public static final String PROPERTYNAME_RELEASE_DATE = "releaseDate";
    public static final String PROPERTYNAME_REVIEW       = "review";
    public static final String PROPERTYNAME_TITLE        = "title";
    
    
    // Instance Fields ********************************************************
    
    private String  title;
    private String  authors;
    private Date    releaseDate;
    private String  review;
    private boolean available;
    private int     price;    // in cents


    // Instance Creation ******************************************************
    
    /**
     * Constructs an empty book with an initial blank review text.
     */
    public MusicCD() {
        review = ""; 
    }
    
    
    private MusicCD(String title, String authors, String releaseDate, boolean available,
            int price, String review) {
        setTitle(title);
        setAuthors(authors);
        DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT, Locale.ENGLISH);
        try {
            setReleaseDate(dateFormat.parse(releaseDate));
        } catch (Exception e) {
            System.out.println("Can't parse '" + releaseDate + "'.");
        }
        setAvailable(available);
        setPrice(price);
        setReview(review);
    }
    
    
    // Creating Example Instances *********************************************
    
    private static MusicCD createExample1() {
        return new MusicCD(
                "Pieces of Africa", 
                "Kronos Quartett",
                "12/5/1992",
                true,
                3500,
                "Contemporary classic.");
    }
    
    private static MusicCD createExample2() {
        return new MusicCD(
                "Aria", 
                "Gianna Nannini",
                "3/17/2002",
                false,
                1800,
                "");
    }
    
    private static MusicCD createExample3() {
        return new MusicCD(
                "The Caterpillar Songs - Canciones de Isaac Albeniz", 
                "Marina Pardo, Rosa Torres-Pardo",
                "4/20/2002",
                true,
                3200,
                "");
    }
    
    private static MusicCD createExample4() {
        return new MusicCD(
                "trampin'", 
                "Patty Smith",
                "6/26/2004",
                true,
                2037,
                "");
    }
    
    

    // Accessors **************************************************************
    
    
    /**
     * Returns this book's author or authors. For example "Kathy Walrath"
     * for a single author, or "Kevin Mullet, Darrel Sano" for authors. 
     * 
     * @return this book's author or authors.
     */
    public String getAuthors() {
        return authors;
    }
    
    
    /**
     * Answers whether this book is available in the store or not.
     * 
     * @return true if this book is in the store, false if not available
     */
    public boolean isAvailable() {
        return available;
    }
    
    
    /**
     * Returns this book's price in cents. For example 2495 for US$ 24.95.
     * 
     * @return this book's price in cents.
     */
    public int getPrice() {
        return price;
    }
    
    
    /**
     * Returns this book's release date.
     * 
     * @return this book's release date.
     */
    public Date getReleaseDate() {
        return releaseDate;
    }
    
    
    /**
     * Returns this book's optional review. If no review is available
     * a blank string will be returned.
     * 
     * @return the review.
     */
    public String getReview() {
        return review;
    }
    
    
    /**
     * Returns this book's title. For example "The Java Tutorial" or
     * "Designing Visual Interfaces".
     * 
     * @return this book's title.
     */
    public String getTitle() {
        return title;
    }
    
    

    /**
     * Sets a new author or new authors and notifies observers 
     * if the authors changed.
     * 
     * @param authors  The author or authors to set.
     */
    public void setAuthors(String authors) {
        String oldValue = getAuthors();
        this.authors = authors;
        firePropertyChange(PROPERTYNAME_AUTHORS, oldValue, authors);
    }
    

    /**
     * Sets this book's availability and notifies observers 
     * if the availability changed.
     * 
     * @param available   true to indicate that the book is available,
     *     false for absent.
     */
    public void setAvailable(boolean available) {
        boolean oldValue = isAvailable();
        this.available = available;
        firePropertyChange(PROPERTYNAME_AVAILABLE, oldValue, available);
    }
    

    /**
     * Sets this book's price in cents and notifies observers 
     * if the price changed. For example 2495 for US$ 24.95.
     * 
     * @param price   The price to set.
     */
    public void setPrice(int price) {
        int oldValue = getPrice();
        this.price = price;
        firePropertyChange(PROPERTYNAME_PRICE, oldValue, price);
    }
    

    /**
     * Sets this book's release date and notifies observers
     * if the release date changed.
     * 
     * @param releaseDate    The release date to set.
     */
    public void setReleaseDate(Date releaseDate) {
        Date oldValue = getReleaseDate();
        this.releaseDate = releaseDate;
        firePropertyChange(PROPERTYNAME_RELEASE_DATE, oldValue, releaseDate);
    }

    
    /**
     * Sets this book's optional review text and notifies observers
     * if the review text changed.
     * 
     * @param review   The review to set.
     */
    public void setReview(String review) {
        Object oldValue = getReview();
        this.review = review;
        firePropertyChange(PROPERTYNAME_REVIEW, oldValue, review);
    }
    

    /**
     * Sets this book's title and notifies observers 
     * if the title changed.
     * 
     * @param title   The title to set.
     */
    public void setTitle(String title) {
        Object oldValue = getTitle();
        this.title = title;
        firePropertyChange(PROPERTYNAME_TITLE, oldValue, title);
    }
    
    
    // Misc *******************************************************************
    
    /**
     * Returns a string representation of this book
     * that contains the property values in a single text line.
     * 
     * @return a string representation of this book
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer("Book");
        buffer.append(" [title=");
        buffer.append(getTitle());
        buffer.append("; authors=");
        buffer.append(getAuthors());
        buffer.append("; release date=");
        buffer.append(getReleaseDate());
        buffer.append("; price=");
        buffer.append(getPrice());
        buffer.append("; available");
        buffer.append(isAvailable());
        buffer.append("; review=");
        buffer.append(getReview());
        return buffer.toString();
    }

    /**
     * Returns a string representation of this book
     * that contains the property values.
     * 
     * @return a string representation of this book
     */
    public String toWrappedString() {
        StringBuffer buffer = new StringBuffer("Book");
        buffer.append("[\ntitle=");
        buffer.append(getTitle());
        buffer.append(";\nauthors=");
        buffer.append(getAuthors());
        buffer.append(";\nrelease date=");
        buffer.append(getReleaseDate());
        buffer.append(";\nprice=");
        buffer.append(getPrice());
        buffer.append(";\navailable");
        buffer.append(isAvailable());
        buffer.append(";\nreview=");
        buffer.append(getReview());
        return buffer.toString();
    }

}
