package de.kleopatra.forms.examples;

import java.awt.Container;
import java.util.Arrays;

import javax.swing.*;

import de.kleopatra.forms.description.*;
import de.kleopatra.forms.view.builder.*;
import de.kleopatra.support.KleoUtilities;

/** Example how to change Styles dynamically. <p>
 *
 *  This is helpful if you need alignment across different
 *  containers, f.i. in groupboxes or the tabs of a
 *  JTabbedPane. The usual approach to prevent "jumping"
 *  columns is to use a Constant/BoundedSize in the
 *  corresponding ColumnSpec.
 *
 *  This size might be inappropriate at runtime if
 *  the columns' contents can change dynamically,
 *  f.i. by supporting localized labels. <p>
 *
 *  The SwingEmpire approach in this case is to
 *  design the form with a custom (column content) style,
 *  and store the edited StyleMap as a application resource.
 *  At runtime after loading the custom StyleMap
 *  (best before loading the forms) the custom style can
 *  be changed on-the-fly. The changes are effective
 *  after rebuilding the form(s). <p>
 *
 *  In this example the first label column of the groupboxes
 *  is built with a custom style GROUP_BOX_LABEL_KEY which can
 *  be edited in the control text field. Apply will try to update
 *  the style, rebuild and pack the form. <p>
 *
 *  PENDING: actually load a localized Presentation.
 *
 *  @version $Revision: 1.4 $ $Date: 2004/09/30 10:16:41 $
 *  @author (C) 2004 Jeanette Winzenburg
 *
 */
public class QuickstartStyled extends AbstractForm {

  // layout resources
  private static final String FORM_RESOURCE = "labelstylemain";
  private static final String CONTROL_RESOURCE = "labelstylecontrol";

//  private static final String GENERAL_BOX_RESOURCE = "labelstylegeneralbox";
//  private static final String PROPELLER_BOX_RESOURCE =
//    "labelstylepropellerbox";
//  private static final String ALTERNATIV_GENERAL_BOX_PRESENTATION =
//    "labelstylegeneralbox_de";
//  private static final String ALTERNATIV_PROPELLER_BOX_PRESENTATION =
//    "labelstylepropellerbox_de";
  // style resources
  private static final String GROUPBOX_STYLEMAP = "groupboxstylemap";
  private static final String GROUPBOX_LABEL_TYPE_KEY = "groupboxLabel";

  // editable data
  private Style style;

//--------------------- constructor
  public QuickstartStyled() {
    prepareUI();
  }

//-------------------- Form implementation
  public String getResource() {
    return FORM_RESOURCE;
  }

  protected String getButtonResource() {
    return CONTROL_RESOURCE;
  }

  protected void bind(JComponent form) {
    initTextField(form, "company", "Swing Empire", true);
    lookupStyle();
    updateData(form);
  }

//-----------------------accessing style data
  public Style getStyle() {
    return style;
  }

  public String getTypeKey() {
    return (style != null) ? style.getLogicalType().toString() : null;
  }

  public String getEncoding() {
    return (style != null) ? style.getEncodedString() : null;
  }

//-----------------------action callbacks
  public void apply() {
    JTextField field = KleoUtilities.findTextField(form, "encoding");
    if ((field == null) || (style == null)) {
      return;
    }
    String encoding = field.getText();
    try {
      StyleMap map = getCurrentStyleMap();
      map.addColumnContentStyle(style.getLogicalType(), encoding,
          style.getDescription());
      lookupStyle();
      reloadForm();
    } catch (IllegalArgumentException e) {
      e.printStackTrace();
    }
  }

  //----------------------- helper
  private void lookupStyle() {
    StyleMap styleMap = getCurrentStyleMap();
    style = styleMap.getColumnContentStyle(GROUPBOX_LABEL_TYPE_KEY);
    updateData(form);
  }

  private void updateData(Container form) {
    if (form == null) {
      return;
    }
    initTextField(form, "styleKey", getTypeKey(), false);
    initTextField(form, "encoding", getEncoding(), true);
  }

  private StyleMap getCurrentStyleMap() {
    StyleMap styleMap = LayoutBuilderFactory.getLayoutFactory().getStyleMap();
    return styleMap;
  }

  private void reloadForm() {
    FormFactory.getFormLoader().rebuildAll(form, getClass(), true);
    SwingUtilities.windowForComponent(form).pack();
  }

//---------------------------init
  private void prepareUI() {

    // set the LF
    System.getProperties().setProperty("swing.defaultlaf",
        "com.jgoodies.plaf.plastic.PlasticLookAndFeel");

    // load and set a custom StyleMap
    StyleMap styleMap =
      FormFactory.getFormLoader().loadStyleMap(GROUPBOX_STYLEMAP, getClass());
    if (styleMap != null) {
      LayoutBuilderFactory.getLayoutFactory().setStyleMap(styleMap);
    }

    // register the actions
    // note that this can be done at any time - even before the
    // the form is loaded. 
    registerHandler(Arrays.asList(new Object[]{ "apply" }), this);
  }

  private void initTextField(Container form, String textID, String textValue,
      boolean editable) {
    JTextField comp = KleoUtilities.findTextField(form, textID);
    if (comp != null) {
      comp.setText(textValue);
      comp.setEditable(editable);
    }
  }

//---------------------------Main
  public static void main(String[] args) {
    QuickstartStyled styled = new QuickstartStyled();
    FormUtils.createFrame(styled, true);
  }
}
